<?php

namespace Litecms\Block\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litecms\Block\Events\BlockWorkflow as BlockWorkflowEvent;
use Litecms\Block\Notifications\BlockWorkflow as BlockWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class BlockWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(BlockWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $block is submitted.
     *
     * @param BlockWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(BlockWorkflowEvent $event)
    {
        $client = $event->$block->client;
        Notification::send($client, new BlockWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $block is published.
     *
     * @param BlockWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(BlockWorkflowEvent $event)
    {
        $client = $event->$block->client;
        Notification::send($client, new BlockWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $block is approved.
     *
     * @param BlockWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(BlockWorkflowEvent $event)
    {
        $client = $event->$block->client;
        Notification::send($client, new BlockWorkflowNotification($event));
    }

    /**
     * Handles the $block workflow event as a listener.
     *
     * @param BlockWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(BlockWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
